# -*- coding: utf-8 -*-
# @Time    : 2022-07-22 18:20
# @Author  : zbmain

import time

import pandas


def delay(second: int):
    time.sleep(second)

    def wrapper(func):
        def inner(*args, **kwargs):
            ret = func(*args, **kwargs)
            return ret

        return inner

    return wrapper


def view_df(df: pandas.DataFrame, head_num: int = 5, tail_num: int = 5, comment: str = 'DataFrame'):
    print('%s row_size:%d' % (comment, len(df.index)))
    return df.head(head_num).append(df.tail(tail_num))


import warnings


def warning_ignored():
    """关闭一些警告(不推荐)"""
    warnings.filterwarnings("ignore", category=DeprecationWarning)
    warnings.filterwarnings("ignore", category=UserWarning)
    warnings.filterwarnings("ignore", category=RuntimeWarning)
    warnings.filterwarnings("ignore", category=FutureWarning)


def set_seed(seed: int = 1990):
    import numpy, random
    random.seed(seed)
    numpy.random.seed(seed)
    try:
        import tensorflow
        tensorflow.random.set_seed(seed)
        tensorflow.set_random_seed(seed)
    except:
        pass

    try:
        import torch
        torch.manual_seed(seed)  # cpu
        torch.cuda.manual_seed(seed)  # gpu
        torch.cuda.manual_seed_all(seed)  # all gpu
    except:
        pass



# -*- coding: utf-8 -*-
# @Time    : 2022-07-20 09:42
# @Author  : zbmain
"""
HOME                -> 环境变量目录
├── MODULE_HOME     -> 模型
├── DATA_HOME       -> 数据集
├── CACHE_HOME      -> 缓存目录
├── CONF_HOME       -> 配置目录
├── TMP_HOME        -> 临时文件
├── LOG_HOME        -> 日志
├── SOURCES_HOME    -> 代码
└── RESOURCES_HOME  -> 资源
"""
import os

HUB_HOME_ENV_KEY = 'WINWIN_HUB_HOME'
"""WINWIN_HOME - 环境变量Key"""
__HUB_HOME_DIR = '.winwin_hub'
"""WINWIN_HOME - 本地目录名"""


def _get_user_home():
    return os.path.expanduser('~')


def _get_hub_home():
    if HUB_HOME_ENV_KEY in os.environ:
        home_path = os.environ[HUB_HOME_ENV_KEY]
        if os.path.exists(home_path):
            if os.path.isdir(home_path):
                return home_path
            else:
                raise RuntimeError('The environment variable WINWIN_HOME {} is not a directory.'.format(home_path))
        else:
            return home_path
    os.environ[HUB_HOME_ENV_KEY] = os.path.join(_get_user_home(), __HUB_HOME_DIR)
    return os.environ[HUB_HOME_ENV_KEY]


def _get_sub_home(directory):
    home = os.path.join(_get_hub_home(), directory)
    os.makedirs(home, exist_ok=True)
    return home


USER_HOME = _get_user_home()
HUB_HOME = _get_hub_home()
MODULE_HOME = _get_sub_home('modules')
DATA_HOME = _get_sub_home('dataset')
CACHE_HOME = _get_sub_home('cache')
CONF_HOME = _get_sub_home('conf')
TMP_HOME = _get_sub_home('tmp')
LOG_HOME = _get_sub_home('log')
SOURCES_HOME = _get_sub_home('sources')
RESOURCES_HOME = _get_sub_home('resources')



# -*- coding: utf-8 -*-
# @Time    : 2022-07-20 20:23
# @Author  : zbmain

from ..utils import func_util
from ..utils import list_util
from ..utils import os_util
from ..utils import pool_util
from ..utils import str_util
from ..utils import support

__all__ = ['support', 'func_util', 'os_util', 'str_util', 'pool_util', 'list_util']



# -*- coding: utf-8 -*-
# @Time    : 2022-07-20 21:09
# @Author  : zbmain
import logging
import os
import shutil
import time

HOME_PATH = os.path.expanduser('~')
"""用户目录"""
CURRENT_ROOT_PATH = os.path.abspath('.')
"""项目跟目录"""


def get_suffix(path):
    """路径后缀"""
    return os.path.splitext(path)[1]


def dir2list_with_suffix(dirpath: str, suffix=('.xlsx', '.csv', '.tsv'), ignoring_hidden_file: bool = True):
    return [os.path.join(dirpath, path) for path in os.listdir(dirpath) if
            get_suffix(path) in suffix and (path[0] != '.' if ignoring_hidden_file else True)]


def mkdir(dirpath: str):
    if not os.path.exists(dirpath):
        os.mkdir(dirpath)


def makedirs(dirpath: str):
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)


def del_dirs(path: str, only_sub_file: bool = True):
    """
    清空目录

    :param path: 待清空的目录
    :param only_sub_file: 只清空子目录和子文件（保留原目录）
    :return:
    """
    if os.path.exists(path):
        if os.path.isdir(path):
            if only_sub_file:
                del_list = os.listdir(path)
                for f in del_list:
                    file_path = os.path.join(path, f)
                    if os.path.isfile(file_path):
                        os.remove(file_path)
                    elif os.path.isdir(file_path):
                        shutil.rmtree(file_path)
            else:
                shutil.rmtree(path)
        else:
            os.remove(path)


def parse_filepath(filepath):
    filepath, fullflname = os.path.split(filepath)
    fname, ext = os.path.splitext(fullflname)
    return filepath, fname, ext


def get_modify_time(file: str, format: str = '%Y/%m/%d %H:%M:%S'):
    """文件名的后四位（是日期）"""
    return time.strftime(format, time.localtime(os.stat(file).st_mtime))


def get_create_time(file: str, format: str = '%Y/%m/%d %H:%M:%S'):
    """文件名的后四位（是日期）"""
    return time.strftime(format, time.localtime(os.stat(file).st_ctime))


def network_state():
    '''ping 网络状态'''
    logging.info('Check Network...')
    result = os.system(u"ping -c2 baidu.com > /dev/null 2>&1")  # 都丢弃，防止输出到控制台
    return result == 0


if __name__ == '__main__':
    print(parse_filepath('/home/deploy/tmp/'))
    print(get_modify_time('/home/deploy/tmp/'))
    print(get_create_time('/home/deploy/tmp/'))
    print(dir2list_with_suffix('/home/deploy/tmp/', ignoring_hidden_file=False))
    del_dirs('/home/deploy/tmp/tmps/', only_sub_file=True)
    print(HOME_PATH)
    print(CURRENT_ROOT_PATH)



# -*- coding: utf-8 -*-
# @Time    : 2022-07-20 20:23
# @Author  : zbmain

# 常用的类型
NUM = '0-9'
LETTERS = 'a-zA-Z'
NORMAL_CHAR = '!,.?！，。？'
SPECIAL_CHAR = '’"#$%&\'()*+-/;；<=>@★☆〇〖〗、【】＜＞《》“”‘’\\[\\\\]^_`{|}~'
CN_Unicode = '\u4E00-\u9FA5'  # 中文
CN_Unicdoe2 = '\u4E00-\u9FEF'  # 中文2


def clearNull(char: str):
    '''防止爬虫抓到特殊隐藏字符'''
    import re
    return re.sub(
        '[\001\002\003\004\005\006\007\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a]+',
        '', char)


def lstrip(text, dirty, num=float('inf')):
    """清除字符结尾"""
    if num > 0 and text:
        if text[0] == dirty:
            return lstrip(text[1:], dirty, num - 1)
    return text


def rstrip(text, dirty, num=float('inf')):
    """清除字符结尾"""
    if num > 0 and text:
        if text[-1] == dirty:
            return rstrip(text[:-1], dirty, num - 1)
    return text


def strip(text, dirty, num=float('inf')):
    """清除首尾字符"""
    if num > 0 and text:
        if text[-1] == dirty:
            return lstrip(rstrip(text[:-1], dirty, num - 1), dirty, num)
    return text


def split(text, split_char: str = ';', prefix_char: str = '', suffix_char: str = '') -> list:
    """
    切分(字符串/列表)

    :param text: 切分对象[str;list]
    :param split_char: text为str时的切分符号
    :param prefix_char: 切分后的前缀字符
    :param suffix_char: 切分后的后缀字符
    :return: 切分后的列表
    """
    queue = type(text) is list and text or strip(text, split_char).split(split_char)
    return [prefix_char + char + suffix_char for char in queue if char]


def remove_invalid_words(x: str):
    return filter(lambda y: y.strip(), map(lambda z: z.strip(), x))


def is_number(s):
    if s:
        try:
            float(s)
            return True
        except ValueError:
            pass
        try:
            import unicodedata
            unicodedata.numeric(s)
            return True
        except (TypeError, ValueError):
            pass
    return False


if __name__ == '__main__':
    print(split('a;b;;c', ';', '_', '.bin'))
    print(split(['a', 'b', 'c'], ';', '_', '.bin'))
    print('a;b;c'.split(';'))
    print(list(remove_invalid_words('a\t \u0020b')))
    print(is_number('123'))
    print(is_number('abc'))
    print(strip(';;;123;;;', ';', 1))



# -*- coding: utf-8 -*-
# @Time    : 2021-07-30 20:14
# @Author  : zbmain
__version__ = '0.0.10'
__author__ = 'winwin'

from . import env
from .utils import support
from .utils import os_util
from .utils.func_util import view_df

__all__ = ['env', 'support', 'os_util', 'view_df']



# -*- coding: utf-8 -*-
# @Time    : 2022-07-22 18:04
# @Author  : zbmain

import json
import logging
import time


class Client:
    def __init__(self, nebula_connect: object):
        self.client = nebula_connect
        self.space = str(nebula_connect.__getattribute__('space'))
        self.comment = str(nebula_connect.__getattribute__('comment'))
        self.heart_beat = int(nebula_connect.__getattribute__('heart_beat'))

    def exec_query(self, query, heart_beat: int = 0):
        resp = self.client.execute(query)
        time.sleep(heart_beat)
        assert resp.is_succeeded(), resp.error_msg()
        return resp

    def test(self, ):
        resp_json = self.client.execute_json("yield 1")
        json_obj = json.loads(resp_json)
        return json.dumps(json_obj, indent=1, sort_keys=True)

    def create_space(self, space: str = '', comment: str = ''):
        space = space or self.space
        comment = comment or self.comment
        assert space
        create_gql = 'CREATE SPACE IF NOT EXISTS %s(vid_type=FIXED_STRING(255)) comment="%s";USE %s;' % (
            space, comment, space)
        logging.info('please wait for create space:"%s"\tGQL:%s' % (space, create_gql))
        return self.exec_query(create_gql, self.heart_beat)

    def use_space(self, space: str = ''):
        space = space or self.space
        assert space
        return self.exec_query('USE %s;' % space)

    def drop_space(self, space: str = '', drop: bool = False):
        """谨慎操作。参数必须再设置[drop=True]才有效. drop默认:False"""
        space = space or self.space
        assert space
        return drop and self.exec_query('DROP SPACE IF EXISTS %s;' % space)

    def del_all_tags(self, space: str = '', drop: bool = False):
        """谨慎操作。参数必须再设置[drop=True]才有效. drop默认:False"""
        space = space or self.space
        assert space
        return drop and self.exec_query('DELETE TAG * FROM "%s";' % space)

    def submit(self, space: str = ''):
        self.use_space(space)
        return self.exec_query('SUBMIT JOB STATS;', self.heart_beat)

    def show_stats(self, space: str = ''):
        self.submit(space)
        return self.exec_query('SHOW STATS;')

    def show_indexs(self, space: str = ''):
        self.use_space(space)
        return self.exec_query('SHOW TAG INDEXES;')

    def show_job(self, space: str = ''):
        self.use_space(space)
        return self.exec_query('SUBMIT JOB COMPACT;')

    def release(self):
        if self.client is not None:
            self.client.release()


import prettytable


def print_resp(resp):
    assert resp.is_succeeded()
    output_table = prettytable.PrettyTable()
    output_table.field_names = resp.keys()
    for recode in resp:
        value_list = []
        for col in recode:
            if col.is_empty():
                value_list.append('__EMPTY__')
            elif col.is_null():
                value_list.append('__NULL__')
            elif col.is_bool():
                value_list.append(col.as_bool())
            elif col.is_int():
                value_list.append(col.as_int())
            elif col.is_double():
                value_list.append(col.as_double())
            elif col.is_string():
                value_list.append(col.as_string())
            elif col.is_time():
                value_list.append(col.as_time())
            elif col.is_date():
                value_list.append(col.as_date())
            elif col.is_datetime():
                value_list.append(col.as_datetime())
            elif col.is_list():
                value_list.append(col.as_list())
            elif col.is_set():
                value_list.append(col.as_set())
            elif col.is_map():
                value_list.append(col.as_map())
            elif col.is_vertex():
                value_list.append(col.as_node())
            elif col.is_edge():
                value_list.append(col.as_relationship())
            elif col.is_path():
                value_list.append(col.as_path())
            elif col.is_geography():
                value_list.append(col.as_geography())
            else:
                print('ERROR: Type unsupported')
                return
        output_table.add_row(value_list)
    print(output_table)



# -*- coding: utf-8 -*-
# @Time    : 2022-07-10 09:42
# @Author  : zbmain
"""
support: 工具
├── 环境配置.env:   文件目录 > 跟目录 > Home环境目录[~/.winwin_hub]
├── mysql connect
├── odps connect
├── holo connect
├── redis connect
├── oss bucket
├── openseach conf
├── nebula connect.session
├── milvus
└── 一些常用功能函数
"""
import csv
import logging
import os
from urllib.parse import urlparse, parse_qs

from dotenv import load_dotenv as __load_dotenv

# 项目.跟目录
CURRENT_DIR = os.path.abspath('.')
# 项目.跟目录创建.cache
CURRENT_CACHE_DIR = os.path.join(CURRENT_DIR, '.cache')
if not os.path.isdir(CURRENT_CACHE_DIR):
    os.mkdir(CURRENT_CACHE_DIR)

try:
    from ..env import __HUB_HOME_DIR
except:
    __HUB_HOME_DIR = '.winwin_hub'


def __get_exists_file(file: str):
    """查找目录优先级: support.py目录 > project跟目录 > HOME环境目录(~/.winwin_hub)"""
    dirs = [os.path.dirname(os.path.abspath(__file__)), os.path.abspath('.'),
            os.path.join(os.path.expanduser('~'), __HUB_HOME_DIR)]
    for _dir in dirs:
        filepath = os.path.join(_dir, file)
        if os.path.exists(filepath):
            return filepath


# 文件目录、项目跟目录、HOME环境目录, 加载.env环境文件
env_file = __get_exists_file('.env')
env_file and __load_dotenv(env_file)
logging.info("current env file used path:[%s]" % env_file)


# MYSQL
def parse_mysql_uri(uri):
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    return {
        'host': parsed_uri.hostname,
        'user': parsed_uri.username,
        'password': parsed_uri.password,
        'port': parsed_uri.port,
        'database': parsed_uri.path[1:],
        'charset': 'utf8mb4' if query.get('charset') is None else query['charset'][0]
    }


def mysql_connect(name="MYSQL_URI"):
    """MYSQL Connect"""
    import pymysql
    return pymysql.connect(**parse_mysql_uri(os.environ[name]), autocommit=True, cursorclass=pymysql.cursors.DictCursor)


# OSS
def parse_oss_uri(uri):
    parsed_uri = urlparse(uri)
    return {
        'endpoint': '{}://{}'.format(parsed_uri.scheme, parsed_uri.hostname),
        'access_key_id': parsed_uri.username,
        'access_key_secret': parsed_uri.password,
        'bucket': parsed_uri.path[1:],
    }


def oss_connect(name="OSS_URI", bucket: str = ''):
    """OSS Bucket"""
    import oss2
    config = parse_oss_uri(os.environ[name])
    return oss2.Bucket(oss2.Auth(config['access_key_id'], config['access_key_secret']), config['endpoint'],
                       bucket or config['bucket'])


def oss_get_object_cache(url, bucket=None, CACHE_DIR: str = CURRENT_CACHE_DIR):
    """下载 OSS文件到缓存目录"""
    bucket = bucket or oss_connect()
    cache_file = os.path.join(CACHE_DIR, os.path.basename(url))
    if not os.path.isfile(cache_file) or os.path.getsize(cache_file) == 0:
        parsed = urlparse(url)
        bucket.get_object_to_file(parsed.scheme and parsed.path[1:] or parsed.path, cache_file)
    return cache_file


def oss_get_object_topath(url, save_path='./', bucket=None):
    """下载 OSS文件到指定目录"""
    bucket = bucket or oss_connect()
    if save_path[-1] == '/' and not os.path.exists(save_path):
        os.makedirs(save_path)
    save_path = not os.path.basename(save_path) and os.path.join(save_path, os.path.basename(url)) or save_path
    if not os.path.isfile(save_path) or os.path.getsize(save_path) == 0:
        parsed = urlparse(url)
        bucket.get_object_to_file(parsed.scheme and parsed.path[1:] or parsed.path, save_path)
    return save_path


def oss_get_object_stream(url, bucket=None):
    """下载 OSS文件流"""
    bucket = bucket or oss_connect()
    parsed = urlparse(url)
    oss_url = parsed.scheme and parsed.path[1:] or parsed.path
    return bucket.get_object(oss_url).read()


def oss_upload_object(file_url, oss_url, bucket=None):
    """上传 本地文件到OSS"""
    bucket = bucket or oss_connect()
    file_name = os.path.basename(file_url)
    parsed = urlparse(oss_url)
    oss_url = parsed.scheme and parsed.path[1:] or parsed.path
    print('oss_url', oss_url)
    oss_url = not os.path.basename(oss_url) and (os.path.join(oss_url, file_name)) or oss_url
    bucket.put_object_from_file(oss_url, file_url)
    return '%s%s/%s' % ('oss://', bucket.bucket_name, oss_url)


def oss_file2DF(oss_url: str, bucket=None, sep: str = None):
    """注意文件必须是表格文件"""
    import pandas
    suffix = os.path.splitext(oss_url)[1]
    content = oss_get_object_stream(oss_url, bucket)
    if suffix == '.xlsx':
        return pandas.read_excel(content)
    else:
        sep = sep or (',' if suffix == '.csv' else '\t' if suffix == '.tsv' else ',')
        lines = str(content.decode()).strip().split('\n')
        header, data = lines[0].split(sep), list(map(lambda x: str(x).split(sep), lines[1:]))
        return pandas.DataFrame(data, columns=header)


# ODPS
ODPS_TUNNEL, ODPS_LIMIT = True, False


def parse_odps_uri(uri):
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    ODPS_TUNNEL = True if query.get('tunnel') is None else query.get('tunnel')[0]
    ODPS_LIMIT = False if query.get('limit') is None else query.get('limit')[0]
    return {
        'endpoint': '{}://{}'.format(parsed_uri.scheme, os.path.join(parsed_uri.hostname, parsed_uri.path[1:])),
        'access_id': parsed_uri.username,
        'secret_access_key': parsed_uri.password,
        'project': query.get('project') and query.get('project')[0] or 'zhidou_hz'
    }


def odps_connect(name="ODPS_URI"):
    """ODPS / Maxcompute Connect"""
    from odps import ODPS
    return ODPS(**parse_odps_uri(os.environ[name]))


def odps_download_with_sql(sql, save_path, connect=None, sep=','):
    """通过SQL下载全量数据"""
    connect = connect or odps_connect()
    save_path = os.path.join(CURRENT_DIR, save_path)
    if sql.endswith('.sql'):
        with open(os.path.join(CURRENT_DIR, sql), 'r') as f:
            fsql = f.read()
    else:
        fsql = sql
    with connect.execute_sql(fsql).open_reader(tunnel=ODPS_TUNNEL, limit=ODPS_LIMIT) as reader:
        headers = reader.schema.names
        with open(save_path, 'w', encoding='utf8') as writefile:
            csv_writer = csv.writer(writefile, delimiter=sep)
            csv_writer.writerow(headers)
            for record in reader:
                csv_writer.writerow(dict(record).values())


# OpenSearch
def parse_ops_uri(uri):
    '''
    security_token有值,type=sts;
    security_token无值,type=access_key;
    '''
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    security_token = query.get('security_token') and query.get('security_token')[0]
    assert query.get('app_name'), """opensearch-uri have to set 'app_name'"""
    return {
        'protocol': parsed_uri.scheme,
        'endpoint': parsed_uri.hostname,
        'access_key_id': parsed_uri.username,
        'access_key_secret': parsed_uri.password,
        'type': security_token and 'sts' or 'access_key',
        'security_token': security_token,
        'app_name': query.get('app_name') and query.get('app_name')[0] or ''
    }


def ops_conf(name="OPS_URI"):
    """OpenSearch Conf"""
    return parse_ops_uri(os.environ[name])


# Hologres
def parse_holo_uri(uri):
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    return {
        'host': parsed_uri.hostname,
        'user': parsed_uri.username,
        'password': parsed_uri.password,
        'port': parsed_uri.port,
        'dbname': query.get('dbname') and query.get('dbname')[0] or 'zhidou_hz'
    }


def holo_connect(name="HOLO_URI"):
    """Holo Connect"""
    import psycopg2
    return psycopg2.connect(**parse_holo_uri(os.environ[name]))


# Redis
def parse_redis_uri(uri):
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    return {
        'host': parsed_uri.hostname,
        'username': parsed_uri.username,
        'password': parsed_uri.password,
        'port': parsed_uri.port,
        'db': query.get('db') and int(query.get('db')[0]) or 0,
        'decode_responses': query.get('decode_responses') and bool(query.get('decode_responses')[0]) or False
    }


__redis_pool = None


def redis_connect(name="REDIS_URI"):
    """Redis Connect"""
    import redis
    global __redis_pool
    redis_uri = parse_redis_uri(os.environ[name])
    if not (__redis_pool and __redis_pool.connection_kwargs['host'] == redis_uri['host'] \
            and __redis_pool.connection_kwargs['db'] == redis_uri['db']):
        __redis_pool = redis.ConnectionPool(**redis_uri)
    return redis.StrictRedis(connection_pool=__redis_pool)


def parse_nebula_uri(uri=None):
    parsed_uri = urlparse(uri)
    query = parse_qs(parsed_uri.query)
    servers = [(parsed_uri.hostname, parsed_uri.port)] + [(server.split(':')[0], int(server.split(':')[1])) for server
                                                          in query.get('server', [])]
    return {
        'servers': servers,
        'username': parsed_uri.username,
        'password': parsed_uri.password,
        'space': query.get('space') and query.get('space')[0] or '',
        'comment': query.get('comment') and query.get('comment')[0] or '',
        'heart_beat': query.get('heart_beat') and int(query.get('heart_beat')[0]) or 10,
    }


__nebula_pool = None


def nebula_connect(name='NEBULA_URI'):
    uri = parse_nebula_uri(os.environ[name])
    from nebula3.gclient.net import ConnectionPool
    from nebula3.Config import Config
    global __nebula_pool
    if not __nebula_pool:
        __nebula_pool = ConnectionPool()
        __nebula_pool.init(uri['servers'], Config())
    session = __nebula_pool.get_session(uri['username'], uri['password'])
    session.space, session.comment, session.heart_beat = uri['space'], uri['comment'], uri['heart_beat']
    return session


def parse_milvus_uri(uri=None):
    parsed_uri = urlparse(uri)
    return {
        'alias': parsed_uri.username,
        'host': parsed_uri.hostname,
        'port': parsed_uri.port
    }


def milvus_connect(name='MILVUS_URI'):
    from pymilvus import connections
    connections.connect(**parse_milvus_uri(os.environ[name]))



# -*- coding: utf-8 -*-
# @Time    : 2022-07-20 20:41
# @Author  : zbmain
class ObjectPool(object):
    def __init__(self, Cls):
        super(ObjectPool, self).__init__()
        self.Cls = Cls
        self.__storage__ = {}

    def _put(self, obj: object, key=None, froce=True):
        key = key or (hasattr(obj, 'name') and obj.__getattribute__('name')) or ''
        if froce or self.__storage__.get(key) is not None:
            self.__storage__[key] = obj

    def set(self, key, *args, **kwargs):
        _obj = self.__storage__.get(key)
        if _obj:
            _obj.__init__(*args, **kwargs)
        else:
            _obj = self.Cls(*args, **kwargs)
            self._put(_obj, key)
        return _obj

    def get(self, key, *args, **kwargs):
        """
        pool get instance of Cls

        :param key: unique key
        :param args: class args param
        :param kwargs: class kwargs param
        :return: unique key instance
        """
        return self.set(key, *args, **kwargs)


import threading


def Singleton(cls):
    '''普通单例装饰器，不支持多线程 @Singleton'''
    _instance = {}

    def _singleton(*args, **kargs):
        if cls not in _instance:
            _instance[cls] = cls(*args, **kargs)
        return _instance[cls]

    return _singleton


class SingletonType(type):
    '''元类单例模式，支持多线程 Class(metaclass=SingletonType])'''
    _instance_lock = threading.Lock()

    def __call__(cls, *args, **kwargs):
        if not hasattr(cls, "_instance"):
            with SingletonType._instance_lock:
                if not hasattr(cls, "_instance"):
                    cls._instance = super(SingletonType, cls).__call__(*args, **kwargs)
        return cls._instance


if __name__ == '__main__':
    class A:
        def __init__(self, name=None):
            self.name = name


    pool = ObjectPool(A)
    print(pool.get('win', 'winwin').name)
    print(pool.__storage__.keys())
    print(pool.get('win', 'winers').name)
    print(pool.__storage__.keys())
    print('ok')



# -*- coding: utf-8 -*-
# @Time    : 2021-07-30 20:14
# @Author  : zbmain
import os

import setuptools

here = os.path.dirname(__file__)

with open("README.md", "r") as fh:
    long_description = fh.read()


def parse_requirements(fname):
    with open(file=fname, mode='r', encoding="utf-8-sig") as f:
        requirements = f.readlines()
    return requirements


here and os.chdir(here)

setuptools.setup(
    name='winwin',
    version='0.0.10',
    packages=setuptools.find_packages(
        where='.',
        exclude=('.cache', 'tests*', 'build*', '*.egg*', 'dist*')
    ),
    package_dir={'winwin': 'winwin'},
    url='https://pypi.org/project/winwin/',
    license='Apache 2.0',
    author='zhaobin',
    author_email='zhaobin@banmahui.cn',
    description='',
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=['python-dotenv', 'oss2', 'redis', 'pymysql', 'openpyxl', 'pyodps', 'psycopg2-binary'],
    python_requires='>=3.7',
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    # .egg
    zip_safe=True
)
